﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

using Google.Cloud.Vision.V1;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 이미지 주석자 클라이언트
        /// </summary>
        private ImageAnnotatorClient client;

        /// <summary>
        /// 파일 열기 대화 상자
        /// </summary>
        private OpenFileDialog openFileDialog;

        /// <summary>
        /// 로컬 객체 주석 리스트
        /// </summary>
        private IReadOnlyList<LocalizedObjectAnnotation> annotationList = null;

        /// <summary>
        /// 소스 이미지
        /// </summary>
        private System.Drawing.Image sourceImage = null;

        /// <summary>
        /// 기준 점수
        /// </summary>
        private float baseScore = 0.8f;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 이미지 주석자 클라이언트를 설정한다.

            this.client = ImageAnnotatorClient.Create();

            #endregion
            #region 파일 열기 대화 상자를 설정한다.

            this.openFileDialog = new OpenFileDialog();

            this.openFileDialog.Filter      = "이미지 파일|*.bmp;*.jpg;*.jpeg;*.jpe;*.gif;*.tif;*.tiff;*.png";
            this.openFileDialog.FilterIndex = 1;

            #endregion
            #region 픽처 박스를 설정한다.

            this.pictureBox.SizeMode = PictureBoxSizeMode.Normal;

            #endregion
            #region 이벤트를 설정한다.

            this.imageButton.Click  += imageButton_Click;
            this.updateButton.Click += updateButton_Click;
            this.pictureBox.Paint   += pictureBox_Paint;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 이미지 버튼 클릭시 처리하기 - imageButton_Click(sender, e)

        /// <summary>
        /// 이미지 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void imageButton_Click(object sender, EventArgs e)
        {
            if(this.openFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            this.imageButton.Enabled      = false;
            this.baseScoreTextBox.Enabled = false;

            try
            {
                this.pictureBox.Image = null;

                this.sourceImage = null;

                string imageFilePath = this.openFileDialog.FileName;

                this.annotationList = await client.DetectLocalizedObjectsAsync(Image.FromFile(imageFilePath));

                this.sourceImage = System.Drawing.Image.FromFile(imageFilePath);

                SetBaseScore();

                this.pictureBox.Image = this.sourceImage;

                this.pictureBox.Refresh();
            }
            finally
            {
                this.imageButton.Enabled      = true;
                this.baseScoreTextBox.Enabled = true;
            }
        }

        #endregion
        #region 업데이트 버튼 클릭시 처리하기 - updateButton_Click(sender, e)

        /// <summary>
        /// 업데이트 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void updateButton_Click(object sender, EventArgs e)
        {
            SetBaseScore();

            this.pictureBox.Refresh();
        }

        #endregion
        #region 픽처 박스 페인트시 처리하기 - pictureBox_Paint(sender, e)

        /// <summary>
        /// 픽처 박스 페인트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pictureBox_Paint(object sender, PaintEventArgs e)
        {
            if(this.sourceImage == null)
            {
                return;
            }

            if(this.annotationList == null || this.annotationList.Count == 0)
            {
                return;
            }

            System.Drawing.Graphics graphics = e.Graphics;

            int imageWidth  = sourceImage.Width;
            int imageHeight = sourceImage.Height;

            int x1;
            int y1;

            int x2;
            int y2;

            NormalizedVertex vertex1;
            NormalizedVertex vertex2;

            foreach(LocalizedObjectAnnotation annotation in this.annotationList)
            {
                if(annotation.Score < this.baseScore)
                {
                    continue;
                }

                var vertextList = annotation.BoundingPoly.NormalizedVertices;

                vertex1 = vertextList[0];

                x1 = (int)(imageWidth  * vertex1.X) + 5;
                y1 = (int)(imageHeight * vertex1.Y) + 5;

                graphics.DrawString
                (
                    $"{annotation.Name} ({annotation.Score.ToString("#,##0.##")})",
                    Font,
                    System.Drawing.Brushes.Red,
                    x1,
                    y1
                );

                for(int i = 0; i < vertextList.Count; i++)
                {
                    if(i == vertextList.Count - 1)
                    {
                        vertex1 = vertextList[i];
                        vertex2 = vertextList[0];
                    }
                    else
                    {
                        vertex1 = vertextList[i    ];
                        vertex2 = vertextList[i + 1];
                    }

                    x1 = (int)(imageWidth  * vertex1.X);
                    y1 = (int)(imageHeight * vertex1.Y);
                    
                    x2 = (int)(imageWidth  * vertex2.X);
                    y2 = (int)(imageHeight * vertex2.Y);

                    graphics.DrawLine(System.Drawing.Pens.Red, x1, y1, x2, y2);
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 기준 점수 설정하기 - SetBaseScore()

        /// <summary>
        /// 기준 점수 설정하기
        /// </summary>
        private void SetBaseScore()
        {
            try
            {
                this.baseScore = Convert.ToSingle(this.baseScoreTextBox.Text);

                if(this.baseScore < 0f)
                {
                    this.baseScore = 0f;

                    this.baseScoreTextBox.Text = "0.0";
                }
            }
            catch
            {
                this.baseScore = 0.8f;

                this.baseScoreTextBox.Text = "0.8";
            }
        }

        #endregion
    }
}